﻿// Copyright:     (C) 2011 by Zachary Gramana
// Author:        Zachary Gramana
// Version:       0.1
// Description:   Invokes hg from inside a C# application.
// StartedOn:     4/29/2011
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using IronPython.Hosting;
using IronPython.Runtime;
using IronPython.Runtime.Exceptions;
using Microsoft.Scripting.Hosting;

namespace IronMercurialTest
{
	class Program
	{
		public static void Main(string[] rawArgs)
		{
			ScriptEngine ipy = null;
			Int32 returnCode = 0;

			Console.WriteLine("Running 'hg'...\r\n\r\n");
			try
			{
				var outputPath = Path.Combine(Environment.CurrentDirectory, "pcg.rolelayer");

				// Create the "commandline args" for the Python script, aka hg.py.
				var args = new List { Path.Combine(Environment.CurrentDirectory, @"Scripts\hg.py"), "clone", "https://bitbucket.org/zgramana/pcg.rolelayer", outputPath };
				var configOptions = new Dictionary<string, object> { {"FullFrames", true}, {"CompilationThreshold", 1} };
#if DEBUG
				configOptions["Debug"] = true;	// hits breakpoints in python code as well.				
#else
				configOptions["Debug"] = false;
				configOptions["ShowClrExceptions"] = true; // will print exception to console.
#endif
				args.Add("-v");					// Enables verbose output.
				args.Add("--debug");			// tells hg to log debugging output to console.

				ipy = Python.CreateEngine(configOptions);
				var hg = ipy.CreateScriptSourceFromFile(args[0].ToString());

				var sys = ipy.GetSysModule();
				sys.SetVariable("argv", args);

				// Set Python search paths.
				var pythonPaths = new List<String>();

				//var paths = Environment.GetEnvironmentVariable("IRONPYTHONPATH")
				var paths = new StringBuilder(Environment.CurrentDirectory + ";" );
				paths.Append(Path.Combine(Environment.CurrentDirectory, "Lib") + ";");
				paths.Append(Path.Combine(Environment.CurrentDirectory, "Scripts") + ";");
				paths.Append(Path.Combine(Environment.CurrentDirectory, @"Lib\site-packages") + ";");

				paths.ToString().Split(';').ToList().ForEach(p => { if (!String.IsNullOrWhiteSpace(p)) pythonPaths.Add(p); });

				var iterationsLeft = 2;

				ipy.SetSearchPaths(pythonPaths);

				do
				{
					if (Directory.Exists(outputPath))
					{
						Console.Write("\r\n >>> Removing {0}...", outputPath);
						Directory.Delete(outputPath, true);
						Console.Write("Done!\r\n\r\n");
					}

					var start = DateTime.Now.Ticks;

					// Run the script as a program.
					returnCode = hg.ExecuteProgram();

					var end = DateTime.Now.Ticks;
					Console.WriteLine("\r\n\r\n >>> Iteration {0} took {1} seconds.", 2 - iterationsLeft, TimeSpan.FromTicks(end - start).TotalSeconds);
				}	while (iterationsLeft-- > 0);

			}
			catch (SystemExitException)
			{
				Console.WriteLine("\r\nThe script has finished running.\r\n");
			}
			catch (Exception ex)
			{
				if (ipy != null)
				{
					var exception = ipy.GetService<ExceptionOperations>();
					Console.WriteLine(exception.FormatException(ex));
				}

				Console.WriteLine(String.Format("{1}{0}{1}", ex.Message, "\r\n\r\n"));
			}
			Console.WriteLine(String.Format("\r\n >>> Result Code: {0}\r\n", returnCode));
			Console.Write("\r\n\r\n <<< Press any key to continue . . . ");
			Console.ReadKey(true);
		}
	}
}