﻿using System;
using System.Diagnostics;
using System.IO;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using PCG.Mercurial;

namespace PCG.Mercurial.Tests
{
    ///<summary>
    ///  This is a test class for RepositoryManagerTest and is intended
    ///  to contain all RepositoryManagerTest Unit Tests
    ///</summary>
    [TestClass]
    public class RepositoryManagerTest
    {
        private const string PreconditionCheckFailedNoChanges = "Precondition check failed.  No files were added so there was nothing to commit.";

        ///<summary>
        ///  Gets or sets the test context which provides
        ///  information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext { get; set; }

        #region Additional test attributes

        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //

        #endregion

        ///<summary>
        ///  A test for <see cref="RepositoryManager"/> Constructor
        ///</summary>
        [TestMethod]
        public void RepositoryManagerConstructorTest()
        {
            var target = new RepositoryManager();
        }

        ///<summary>
        ///  A test for AddFile
        ///</summary>
        [TestMethod]
        public void AddFileTest()
        {

            var repoInfo = new DirectoryInfo(String.Format(@"c:\temp\RepoTest-{0}", DateTime.Now.Ticks));
            repoInfo.Create();
            var target = RepositoryManager.Create(repoInfo);
            var newFile = new FileInfo(String.Format(@"{0}\TestFile{1}", repoInfo.FullName, DateTime.Now.Ticks));
            File.WriteAllText(newFile.FullName ,"This is part of a test of RepositoryManager's AddFileCommand.");
            HgResult actual;
            actual = target.Add(repoInfo, new[] { newFile });
            Assert.AreEqual(HgResult.Success, actual);
        }

        ///<summary>
        ///  A test for Create
        ///</summary>
        [TestMethod]
        public void CreateTest()
        {
            DirectoryInfo repositoryLocation = null; // TODO: Initialize to an appropriate value
            string filePathToExecutable = string.Empty; // TODO: Initialize to an appropriate value
            RepositoryManager expected = null; // TODO: Initialize to an appropriate value
            RepositoryManager actual;
            actual = RepositoryManager.Create(repositoryLocation, filePathToExecutable);
            Assert.AreEqual(expected, actual);
        }

        ///<summary>
        ///  A test for InitializeRepository
        ///</summary>
        [TestMethod]
        public void InitializeRepositoryTest()
        {
            var repoInfo = new DirectoryInfo(String.Format(@"c:\temp\RepoTest-{0}", DateTime.Now.Ticks));
            repoInfo.Create();
            var target = RepositoryManager.Create(repoInfo) ; 
            HgResult actual;
            actual = target.Init(repoInfo);
            Assert.AreEqual(HgResult.Success, actual);
        }
        
        ///<summary>
        ///  A test for RunSynchronousCommand
        ///</summary>
        [TestMethod]
        [DeploymentItem("PCG.Mercurial.dll")]
        public void RunSynchronousCommandTest()
        {
            var target = new Mercurial.RepositoryManager_Accessor(); // TODO: Initialize to an appropriate value
            HgCommandBase command = null; // TODO: Initialize to an appropriate value
            Process expected = null; // TODO: Initialize to an appropriate value
            Process actual;
            actual = target.RunSynchronousCommand(command);
            Assert.AreEqual(expected, actual);
        }

        ///<summary>
        ///  A test for SearchFileSystemForHgPath
        ///</summary>
        [TestMethod]
        [DeploymentItem("PCG.Mercurial.dll")]
        public void SearchFileSystemForHgPathTest()
        {
            string expected = string.Empty; // TODO: Initialize to an appropriate value
            string actual;
            actual = Mercurial.RepositoryManager_Accessor.SearchFileSystemForHgPath();
            Assert.AreEqual(expected, actual);
        }

        ///<summary>
        ///  A test for SearchPathForHg
        ///</summary>
        [TestMethod]
        [DeploymentItem("PCG.Mercurial.dll")]
        public void SearchPathForHgTest()
        {
            string expected = string.Empty; // TODO: Initialize to an appropriate value
            string actual;
            actual = Mercurial.RepositoryManager_Accessor.SearchPathForHg();
            Assert.AreEqual(expected, actual);
        }

        ///<summary>
        ///  A test for DefaultExecutableName
        ///</summary>
        [TestMethod]
        public void DefaultExecutableNameTest()
        {
            var target = new RepositoryManager(); // TODO: Initialize to an appropriate value
            string actual;
            actual = target.DefaultExecutableName;
        }

        ///<summary>
        ///  A test for ExecutableInfo
        ///</summary>
        [TestMethod]
        public void ExecutableInfoTest()
        {
            var target = new RepositoryManager(); // TODO: Initialize to an appropriate value
            FileInfo expected = null; // TODO: Initialize to an appropriate value
            FileInfo actual;
            target.ExecutableInfo = expected;
            actual = target.ExecutableInfo;
            Assert.AreEqual(expected, actual);
        }

        ///<summary>
        ///  A test for Repository
        ///</summary>
        [TestMethod]
        public void RepositoryTest()
        {
            var target = new RepositoryManager(); // TODO: Initialize to an appropriate value
            DirectoryInfo expected = null; // TODO: Initialize to an appropriate value
            DirectoryInfo actual;
            target.RepositoryLocation = expected;
            actual = target.RepositoryLocation;
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for Add. TODO.ZJG: The <see cref="HgResult"/> pretty much always comes back as success.
        ///</summary>
        [TestMethod()]
        public void AddTest()
        {
            var repoInfo = new DirectoryInfo(String.Format(@"c:\temp\RepoTest-{0}", DateTime.Now.Ticks));
            repoInfo.Create();
            var target = RepositoryManager.Create(repoInfo);

            // Initialize the repository.
            target.Init(repoInfo);

            var newFile = new FileInfo(String.Format(@"{0}\TestFile{1}", repoInfo.FullName, DateTime.Now.Ticks));
            File.WriteAllText(newFile.FullName, "This is part of a test of RepositoryManager's AddFileCommand.");
            HgResult actual;
            actual = target.Add(repoInfo, new[] { newFile });
            Assert.AreEqual(HgResult.AddFileSuccess, actual);
        }

        /// <summary>
        ///A test for adding files to a repository.  This test will create a subdirectory of ten files within the repository's root folder.
        /// It will then create a nested subdirectory that also contains ten files.  The add command will only add the ten files in the child directory,
        /// not not the ten grandchild files.
        ///</summary>
        [TestMethod()]
        public void AddSubDirectoryTest()
        {
            var repoInfo = new DirectoryInfo(String.Format(@"c:\temp\RepoTest-{0}", DateTime.Now.Ticks));
            repoInfo.Create();
            var target = RepositoryManager.Create(repoInfo);

            // Initialize the repository.
            target.Init(repoInfo);

            // Create the child directory.
            var directory = new DirectoryInfo(String.Format(@"{0}\SubDirectory-{1}", repoInfo.FullName, DateTime.Now.Ticks));
            directory.Create();

            // Create the grandchild directory.
            var nestedDirectory = new DirectoryInfo(String.Format(@"{0}\NestedDirectory-{1}", directory.FullName, DateTime.Now.Ticks));
            nestedDirectory.Create();

            // Create some test files.
            // Files in the nested folder should
            // not be added to the repo.
            for (int i = 0; i < 10; i++)
            {
                var newFile = new FileInfo(String.Format(@"{0}\TestFile{1}", directory.FullName, i));
                File.WriteAllText(newFile.FullName, "This is part of a test of RepositoryManager's AddFileCommand.");

                var newNestedFile = new FileInfo(String.Format(@"{0}\NestedFile{1}", nestedDirectory.FullName, i));
                File.WriteAllText(newNestedFile.FullName, "This is part of a test of RepositoryManager's AddFileCommand.");
            }

            HgResult actual;
            actual = target.Add(repoInfo, directory, false);

            Assert.AreEqual(HgResult.AddFileSuccess, actual);
        }

        /// <summary>
        ///A test for adding files to a repository.  This test will create a subdirectory of ten files within the repository's root folder.
        /// It will then create a nested subdirectory that also contains ten files.  The add command will only add the ten files in the child directory,
        /// not not the ten grandchild files.
        ///</summary>
        [TestMethod()]
        public void CommitTest()
        {
            var repoInfo = new DirectoryInfo(String.Format(@"c:\temp\RepoTest-{0}", DateTime.Now.Ticks));
            repoInfo.Create();
            var target = RepositoryManager.Create(repoInfo);

            // Initialize the repository.
            target.Init(repoInfo);

            // Create the child directory.
            var directory = new DirectoryInfo(String.Format(@"{0}\SubDirectory-{1}", repoInfo.FullName, DateTime.Now.Ticks));
            directory.Create();

            // Create the grandchild directory.
            var nestedDirectory = new DirectoryInfo(String.Format(@"{0}\NestedDirectory-{1}", directory.FullName, DateTime.Now.Ticks));
            nestedDirectory.Create();

            // Create some test files.
            // Files in the nested folder should
            // not be added to the repo.
            for (int i = 0; i < 10; i++)
            {
                var newFile = new FileInfo(String.Format(@"{0}\TestFile{1}", directory.FullName, i));
                File.WriteAllText(newFile.FullName, "This is part of a test of RepositoryManager's AddFileCommand.");

                var newNestedFile = new FileInfo(String.Format(@"{0}\NestedFile{1}", nestedDirectory.FullName, i));
                File.WriteAllText(newNestedFile.FullName, "This is part of a test of RepositoryManager's AddFileCommand.");
            }

            HgResult actual;
            actual = target.Add(repoInfo, directory, false);

            if (actual == HgResult.AddFileSuccess)
            {
                const String commitMessage = @"Testing the CommitCommand object via an automated unit test.";
                actual = target.CommitAllChanges(repoInfo, commitMessage);
                Assert.AreEqual(HgResult.CommitFileSuccess, actual);
            }
            else
            {
                Assert.Fail(PreconditionCheckFailedNoChanges);
            }
        }

        [TestMethod()]
        public void CloneTest()
        {
           var repoManager = IronMercurialRepositoryManager.Create(new DirectoryInfo(@"C:\Temp\AzureOutput"));
           //HgResult result = repoManager.Clone("https://pottman:aldeberan@bitbucket.org/pottman/azurebuildgrid", "pottman", "password");
		   //var result = repoManager.Clone("https://hg01.codeplex.com/db", "pottman", "password");
		   var result = repoManager.Clone("http://bitbucket.org/zgramana/pcg.rolelayer");

           Assert.AreSame(HgResult.CloneSuccess, result);
        }

        [TestMethod()]
        public void IncomingCommandTest()
        {
            RepositoryManager repoManager = RepositoryManager.Create(new DirectoryInfo(@"C:\Temp\AzureOutput"), @"C:\Program Files\Mercurial\hg.exe");
            //repoManager.Clone("https://pottman:aldeberan@bitbucket.org/pottman/onetug.azure", "pottman", "password");

            IncomingCommandResult result = repoManager.Incoming();
            if(result.HasChanges)
            {
                repoManager.Pull();
            }
        }

    }
}