﻿using System;
using System.Diagnostics;
using System.IO;
using System.Text;

namespace PCG.Mercurial
{
    /// <summary>
    /// Provides default implementations for most interface-required members.
    /// </summary>
    public abstract class HgCommandBase : IHgCommand
    {
        #region Constants
        // ####################################################################
        // ###### Constants                                              ######
        // ####################################################################
        internal const String CommandBaseFormat = " {0}";  // {1} = repo full path

        #endregion

        #region Constructors
        // ####################################################################
        // ###### Constructors                                           ######
        // ####################################################################
        /// <summary>
        /// Default constructor.  We must always have a repository location defined,
        /// so we are not providing an empty constructor.
        /// </summary>
        /// <param name="repositoryLocation"></param>
        protected HgCommandBase(DirectoryInfo repositoryLocation)
        {
            Repository = repositoryLocation;
            Debug = false;
            Traceback = false;
            Time = false;
            Profile = false;
            Interactive = false;
            VerboseOutput = true;
        }

        protected HgCommandBase()
        {
            Debug = false;
            Traceback = false;
            Time = false;
            Profile = false;
            Interactive = false;
            VerboseOutput = true;
        }

        #endregion

        #region Properties
        // ####################################################################
        // ###### Properties                                             ######
        // ####################################################################
        public Boolean Debug { get; set; }
        public Boolean Traceback { get; set; }
        public Boolean Time { get; set; }
        public Boolean Profile { get; set; }
        public Boolean Interactive { get; set; }
        public Boolean VerboseOutput { get; set; }

        #endregion

        #region Implementation of IHgCommand
        // ####################################################################
        // ###### Implementation of IHgCommand                           ######
        // ####################################################################
        public DirectoryInfo Repository { get; set; }

        /// <summary>
        /// Returns a properly populated command result based on the results of the command's execution.
        /// </summary>
        /// <param name="hgProcess"></param>
        /// <returns></returns>
        public abstract IHgCommandResult GetCommandResult(Process hgProcess);

        /// <exception cref="NullReferenceException">Repository is required.</exception>
        public override String ToString()
        {
            if (Repository == null)
                throw new NullReferenceException("Repository is required.");

            var hgArgBase = new StringBuilder(String.Format(CommandBaseFormat, Repository.FullName));

            if (VerboseOutput)
                hgArgBase.Append(" -v");

            if (Debug)
                hgArgBase.Append(" --debug");

            if (Traceback)
                hgArgBase.Append(" --traceback");

            if (Time)
                hgArgBase.Append(" --time");

            if (Profile)
                hgArgBase.Append(" --profile");

			//if (!Interactive)
			//    hgArgBase.Append(" -y");

            return hgArgBase.ToString();
        }

        #endregion

    }
}