﻿using System;
using System.Diagnostics;
using System.Diagnostics.Contracts;

namespace PCG.Mercurial
{
    /// <summary>
    /// Defines the interface for command result objects.
    /// </summary>
    public abstract class HgCommandResultBase : IHgCommandResult
    {
        #region Instance Fields
        // ####################################################################
        // ###### Instance Fields                                        ######
        // ####################################################################
        private String _stdout;
        private String _stderr;
        internal readonly Process HgProcess;

        #endregion

        #region Constructors
        // ####################################################################
        // ###### Constructors                                           ######
        // ####################################################################
        /// <summary>
        /// Process must always exist in a result object.  This is why there is no empty constructor.
        /// </summary>
        /// <param name="process"></param>
        protected HgCommandResultBase(Process process)
        {
            Contract.Requires(process != null);

            HgProcess = process;
        }

        #endregion

        #region Properties
        // ####################################################################
        // ###### Properties                                             ######
        // ####################################################################
        public String ResultData
        {
            get
            {
                if (String.IsNullOrEmpty(_stdout))
                {
                    if (HgProcess.HasExited)
                    {
                        var responseBytes = HgProcess.StandardOutput;
                        _stdout = responseBytes.ReadToEnd();
                    }
                }
                return _stdout;
            }
        }
        public String ErrorData
        {
            get
            {
                if (String.IsNullOrEmpty(_stderr))
                {
                    if (HgProcess.HasExited)
                    {
                        var responseBytes = HgProcess.StandardError;
                        _stderr = responseBytes.ReadToEnd();
                    }
                }
                return _stderr;
            }
        }
        public Process Process
        {
            get { return HgProcess; }
        }
        public abstract HgResult ExitCode { get; }

        #endregion
    }
}
