﻿using System;
using System.Diagnostics;
using System.IO;

namespace PCG.Mercurial
{
    public interface IHgCommand
    {
        /// <summary>
        /// Command objects serialize into a command line argument string.
        /// </summary>
        /// <returns></returns>
        String ToString();

        /// <summary>
        /// Enables Hg's --debug option.
        /// </summary>
        Boolean Debug { get; set; }

        /// <summary>
        /// Enables Hg's --traceback option.
        /// </summary>
        Boolean Traceback { get; set; }

        /// <summary>
        /// Enables Hg's --time option, which calculates the amount of time it took to perform the command.
        /// </summary>
        Boolean Time { get; set; }

        /// <summary>
        /// Enables Hg's --profile option.
        /// </summary>
        Boolean Profile { get; set; }

        /// <summary>
        /// By default, we run the Mercurial executable in non-interactive mode (i.e. -y).
        /// Setting Interactive to true removes this argument.  It also means that when
        /// the command is run, it will return a running process that is waiting for a
        /// user input stream on stdin.
        /// </summary>
        Boolean Interactive { get; set; }

        /// <summary>
        /// Many commands return no output by default.  Enabling verbose mode ensures that commands provide some feedback.
        /// </summary>
        Boolean VerboseOutput { get; set; }

        /// <summary>
        /// The location of the repository to be worked on.
        /// </summary>
        DirectoryInfo Repository { get; }

        /// <summary>
        /// Returns a properly populated command result based on the results of the command's execution.
        /// </summary>
        /// <param name="hgProcess"></param>
        /// <returns></returns>
        IHgCommandResult GetCommandResult(Process hgProcess);

    }
}
