﻿using System;
using System.IO;
using IniParser;

namespace PCG.Mercurial
{
	public interface IRepositoryManager
	{
		/// <summary>
		/// The location of the Mercurial repository.
		/// </summary>
		DirectoryInfo RepositoryLocation { get; set; }

		/// <exception cref="FileNotFoundException"><c>FileNotFoundException</c>.</exception>
		IniData HgConfiguration { get; set; }

		/// <summary>
		/// Initializes a new repository in the directory provided.
		/// </summary>
		/// <returns></returns>
		HgResult Init(DirectoryInfo repoInfo);

		HgResult Clone(string repositoryUrl, string username, string password);

		IncomingCommandResult Incoming();

		HgResult Pull();

		/// <summary>
		/// Commits a new changeset to the repository.
		/// </summary>
		/// <returns></returns>
		HgResult CommitAllChanges(DirectoryInfo repoInfo, String commitMessage);

		/// <summary>
		/// Adds (a) new file(s) to a repository.
		/// </summary>
		/// <returns></returns>
		HgResult Add(DirectoryInfo repoInfo, FileInfo[] newFiles);

		/// <summary>
		/// Adds all files in a subdirectory to a repository.
		/// </summary>
		/// <returns></returns>
		HgResult Add(DirectoryInfo repoInfo, DirectoryInfo subDirectory, Boolean recursively);
	}
}