﻿using System;
using System.Diagnostics;
using System.Diagnostics.Contracts;
using System.IO;
using System.Text;

namespace PCG.Mercurial
{
    public class InitCommand : HgCommandBase
    {
        private const String CommandText = "init";

        /// <summary>
        /// Constructor requires repository location, as nothing can be done without it.
        /// </summary>
        /// <param name="repositoryLocation"></param>
        public InitCommand(DirectoryInfo repositoryLocation) : base(repositoryLocation) { }


        #region Overrides of HgCommandBase

        /// <summary>
        /// Returns a properly populated command result based on the results of the command's execution.
        /// Returns null if the process has not yet exited.
        /// </summary>
        /// <param name="hgProcess"></param>
        /// <returns></returns>
        public override IHgCommandResult GetCommandResult(Process hgProcess)
        {
            Contract.Requires(hgProcess != null);


            return hgProcess.HasExited ? new InitCommandResult(hgProcess) : null;
        }

        #endregion


        /// <summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:PCG.Mercurial.IHgCommand"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:PCG.Mercurial.HgCommandBase"/>.
        /// </returns>
        /// <filterpriority>2</filterpriority>
        /// <exception cref="NullReferenceException">Repository is required.</exception>
        public override string ToString()
        {
            if (Repository == null)
                throw new NullReferenceException("Repository is required.");

            var hgArgs = new StringBuilder(CommandText);

            hgArgs.Append(base.ToString());

            return hgArgs.ToString();
        }
    }
}